-- SETTINGS
-- Tails
local TAILS_FLY_COUNT = 3

-- Control types
local cntrltypes = {[0] = "strafe", 
					[1] = "simplestrafe", 
					[2] = "standard", 
					[3] = "simple"}

function A_KRCheckExistence(p)
	if p
	and p.valid
	and p.mo
	and p.mo.valid
		return true
	else
		return false
	end
end

function A_KRPlayerInit(p)
	p.krinit = true
	p.krework = {}
	p.krework.pbuttons = p.cmd.buttons
	p.krework.shieldt = 0
	p.krework.usedshield = false
	if p.krework.freeaim == nil
		p.krework.freeaim = 1
	end
	if p.krework.uncurltoggle == nil
		p.krework.uncurltoggle = true
	end
	if p.krework.superpowers == nil
		p.krework.superpowers = true
	end
	p.mo.krework = {}
	A_KRResetAbilities(p)
	return
end

function A_KRMobjInit(mo)
	mo.krinit = true
	mo.krework = {}
	if mo.skin == "sonic"
		mo.krework.sonic = {}
	end
	if mo.skin == "tails"
		mo.krework.tails = {}
	end
	
	A_KRResetAbilities(mo.player)
end

-- Checks if two things overlap vertically
function A_KRCheckCollisionHeight(a, b)
	if not (a.valid and b.valid)
		return false
	end
	
	if a.z + a.momz + a.height < b.z + b.momz
	or b.z + b.momz + b.height < a.z + a.momz 
		return false
	end
	return true
end

function A_KRCheckNoControl(p)
	if (P_PlayerInPain(p)
	or p.state == S_PLAY_PAIN
	or (p.playerstate == PST_DEAD)
	or p.powers[pw_carry])
		return true
	else
		return false
	end
end

-- Checks if you're one of the game's default characters, so we don't go too crazy with you
function A_KRCheckStockChar(p)
	if p.mo.skin == "sonic"
	or p.mo.skin == "tails"
	or p.mo.skin == "knuckles"
	or p.mo.skin == "amy"
	or p.mo.skin == "fang"
	or p.mo.skin == "metalsonic"
		return true
	else
		return false
	end
end

function A_KRGetCtrlStyle(p)
    -- Returns 3 if "Simple" control scheme is enabled.
    -- Returns 2 if directionchar is set to movement
    -- Returns 1 if directionchar is set to camera, but Simple control scheme is enabled, AKA Analog Strafe
    -- Returns 0 if directionchar is set to camera, AKA Strafe
    return cntrltypes[((p.pflags & PF_ANALOGMODE) and 1 or 0) | ((p.pflags & PF_DIRECTIONCHAR) and 2 or 0)]
end

function A_KRGetMoveDirection(p)
	local direction = p.mo.angle
	
	if A_KRGetCtrlStyle(p) != "simple"
		direction = $ + R_PointToAngle2(0, 0, p.cmd.forwardmove * FRACUNIT, p.cmd.sidemove * -FRACUNIT)
	end
	
	return direction
end

function A_KRResetFlightCount(p)
	p.mo.krework.tails.flycount = TAILS_FLY_COUNT
end

-- Resets a player's abilities.
-- bop is true when this was triggered by hitting a monitor or an enemy
function A_KRResetAbilities(p, bop)
	if not (p and p.mo and p.mo.valid)
		return
	end
	
	local mo = p.mo
	
	-- Reset whether you've curled up mid-air with spin
	-- (Currently only used to eat Tails's remaining fly boosts when you aircurl)
	mo.krework.aircurled = false
	
	-- Reset whether you've used your shield ability
	if not (bop and (p.powers[pw_shield] & SH_NOSTACK) == SH_ATTRACT) -- Don't want attract shield to be chainable
		p.krework.usedshield = false
	end
	
	-- The rest of this stuff is for modified stock characters only
	if not A_KRCheckStockChar(p)
		return
	end
	
	if mo.skin == "sonic"
		mo.krework.sonic = $ or {}
		-- Reset whether Sonic has double-jumped (using PF_THOKKED stops roll on landing)
		mo.krework.sonic.dbljumped = false
		-- Reset how many frames Sonic has charged drop dash
		mo.krework.sonic.dropcharge = $ or 0
	else
		mo.krework.sonic = {}
	end
	
	if mo.skin == "tails"
		mo.krework.tails = $ or {}
		-- Reset how long Tails has been flying
		mo.krework.tails.flytime = 0
		-- Reset how many more times Tails can boost
		A_KRResetFlightCount(p)
	else
		mo.krework.tails = {}
	end
	
	-- Reset the secondjump var
	p.secondjump = 0
	
	-- Reset the 'used ability' flags
	p.pflags = $ & ~PF_THOKKED & ~PF_SHIELDABILITY
end

function A_KRAircurl(p)
	p.mo.state = S_PLAY_JUMP
	p.mo.krework.aircurled = true
	p.mo.krework.tails.flytime = 0
	p.pflags = $ | PF_JUMPED | PF_SPINNING & ~(PF_GLIDING) & ~(PF_THOKKED) & ~(PF_NOJUMPDAMAGE)
	S_StartSound(p.mo, sfx_curl)
	return true
end

function A_KRUncurl(p)
	local mo = p.mo
	
	-- Stop spinning
	p.pflags = $ & ~PF_SPINNING
	
	-- Walk or run
	if FixedHypot(mo.momx, mo.momy) >= p.runspeed
		mo.state = S_PLAY_RUN
	else
		mo.state = S_PLAY_WALK
	end
end