-- SETTINGS
local UNCURL_LOCKTIME = 8 -- Number of frames you must stay curled before you can uncurl
local BOSS_MOMENTUM_PERCENT = 75 -- Reduction factor on boss bouncing
local SHIELD_WINDOW = 7 -- Number of frames you have to let go of mid-air spin to use a shield power

-- Handle pre-frame setup
addHook("PreThinkFrame", function()
	for player in players.iterate()
		if not A_KRCheckExistence(player)
			continue
		end
		
		local mo = player.mo
		
		-- Init
		if player.krinit == nil
			A_KRPlayerInit(player)
		end

		if not (mo.krinit)
			A_KRMobjInit(mo)
		end
		
		-- Dodgy flag, don't use except for certain shields
		if not ((player.powers[pw_shield] & SH_NOSTACK) == SH_BUBBLEWRAP
			or (player.powers[pw_shield] & SH_NOSTACK) == SH_ELEMENTAL
			or (player.powers[pw_shield] & SH_NOSTACK) == SH_FLAMEAURA)
			player.pflags = $ & ~PF_SHIELDABILITY
		end
		
		-- Spindash buffs
		if player.charability2 == CA2_SPINDASH
			-- Buff minimum speed
			player.mindash = max($, 20 * FRACUNIT)
		end
		
		-- Decide whether it's time to use a shield ability
		player.krework.shieldready = false
		if not player.krework.usedshield
		and not (player.cmd.buttons & BT_USE)
		and player.krework.pbuttons & BT_USE
		and player.krework.shieldt > 0
		and player.krework.shieldt <= SHIELD_WINDOW
			player.krework.shieldready = true
			-- Save thokked flag status before shield ability used
			player.krework.thokked = player.pflags & PF_THOKKED
			player.cmd.buttons = $ | BT_USE
			player.pflags = $ & ~PF_USEDOWN & ~PF_THOKKED
		end
		
		-- Reset air abilities on landing, springing
		if P_IsObjectOnGround(mo)
		or (mo.eflags & MFE_SPRUNG)
		or (player.secondjump == UINT8_MAX)
		--or A_KRCheckNoControl(player)
			A_KRResetAbilities(player)
		end
		
		-- Enter jump state mid-air to enable air abilities
		if not (P_IsObjectOnGround(mo)) and mo.state != S_PLAY_JUMP -- You're in the air but not because you jumped
		and (player.cmd.buttons & BT_JUMP) and not (player.pflags & PF_JUMPDOWN) -- You've just pressed jump
		and not A_KRCheckNoControl(player) -- You have control
		and not (player.powers[pw_carry]) -- You're not being carried by something
		and not (mo.skin == "sonic" and mo.krework.sonic.dbljumped) -- If you're Sonic, you haven't used your double jump
		and not (mo.skin == "tails" and (player.bot or mo.krework.tails.flycount > 0)) -- If you're Tails, you haven't burned all your fly boosts
		//and not (player.pflags & PF_THOKKED) -- If anyone else, you haven't used your ability already
		and not (mo.skin == "tails" -- If you're Tails, you're not in the wait between boosts
					and mo.krework.tails.flytime > 0
					and mo.krework.tails.flytime < mo.krework.tails.flylock)
			player.mo.state = S_PLAY_JUMP
			player.pflags = $ | PF_JUMPED
			if player.charability2 == CA2_SPINDASH
				player.pflags = $ & ~PF_NOJUMPDAMAGE
			end
		end
//		print(player.pflags & PF_JUMPED)
		
		-- Untoggled uncurling
		if not player.krework.uncurltoggle -- Uncurling is on release, not toggle
		and P_IsObjectOnGround(mo) -- You're on the ground
		and mo.state == S_PLAY_ROLL -- You're curled
		and not (A_KRCheckNoControl(player)) -- Control is available
		and not (mo.krework.uncurl_lock) -- Uncurling is not locked
		and not (player.cmd.buttons & BT_USE) -- You're not holding spin
			A_KRUncurl(player)
		end
	end
end)

-- Toggled uncurling and aircurling for all characters
addHook("SpinSpecial", function(player)
	for player in players.iterate()
		-- Have to exist to uncurl
		if not A_KRCheckExistence(player)
			return false
		end
		
		local mo = player.mo
		
		-- On ground, attempt toggled uncurl
		if P_IsObjectOnGround(mo)
			if mo.state == S_PLAY_ROLL -- Must be curled
			and player.krework.uncurltoggle -- Uncurl toggle must be on
			and not A_KRCheckNoControl(player) -- Control must be available
			and not mo.krework.uncurl_lock -- Uncurling can't be locked
			and not (player.krework.pbuttons & BT_USE) -- Can't be already holding spin
				A_KRUncurl(player)
				return true
			end
		else
			if not A_KRCheckNoControl(player)
			and (player.pflags & PF_JUMPED)
				player.krework.shieldt = $+1
			end
			
			-- Try to curl in the air
			if player.charability2 == CA2_SPINDASH -- Must be able to spin
			and mo.state != S_PLAY_ROLL -- Can't already be curled up
			and mo.state != S_PLAY_JUMP -- Can't already be curled up
			and not (player.krework.pbuttons & BT_USE) -- Must be pressing spin, not holding it
			and not (player.powers[pw_carry]) -- Can't be being carried by something
				A_KRAircurl(player)
				return true
			end
		end
	end
end)

addHook("ShieldSpecial", function(player)
	-- Use shield power if we decided it's ready earlier
	if player.krework.shieldready
		-- Bubble shield can be chained
		if (player.powers[pw_shield] & SH_NOSTACK) == SH_BUBBLEWRAP 
		or (player.powers[pw_shield] & SH_NOSTACK) == SH_ELEMENTAL 
		or (player.powers[pw_shield] & SH_NOSTACK) == SH_FLAMEAURA
			player.pflags = $ | PF_SHIELDABILITY
		end
		
		if (player.powers[pw_shield] & SH_NOSTACK) != SH_BUBBLEWRAP
			player.krework.usedshield = true
		end
		return false
	-- Otherwise don't
	else
		return true
	end
end)

-- Various controls for all characters
addHook("PlayerThink", function(player)
	-- Big list of reasons to bail out early
	if not A_KRCheckExistence(player)
	or not A_KRCheckStockChar(player)
	or not (player.playerstate == PST_LIVE)
	or player.exiting
	or player.powers[pw_nocontrol]
	or P_PlayerInPain(player)
		return
	end
	
	local mo = player.mo
	
	-- General moveset tweaks
	
	-- Uncurl lock timer
	-- If you're rolling, count down the timer
	if P_IsObjectOnGround(mo) and mo.state == S_PLAY_ROLL 
		if mo.krework.uncurl_lock
			mo.krework.uncurl_lock = max(0, $ - 1)
		end
	-- If you're not, reset the timer
	else
		mo.krework.uncurl_lock = UNCURL_LOCKTIME
	end
	
	-- Full air control when jumping out of a roll for all characters
	if player.pflags & PF_SPINNING and not P_IsObjectOnGround(mo)
		if player.pflags & PF_JUMPED
			player.pflags = $ & ~PF_SPINNING
		end
	end
end)

-- Regain abilities after hitting monitor/enemy/boss
addHook("MobjMoveCollide", function(mo, touched)
	local player = mo.player
	
	if not A_KRCheckExistence(player)
	or not (touched.flags & (MF_MONITOR|MF_ENEMY|MF_BOSS))
	or not (touched.flags & MF_SHOOTABLE)
	or (touched.flags & MF_MISSILE)
	or not A_KRCheckCollisionHeight(mo, touched)
	or not P_PlayerCanDamage(player, touched)
		return
	end
	
	A_KRResetAbilities(player, true)
		
	-- Reduce x and y velocity when bouncing off a boss/facestabber/etc
	if (touched.flags & MF_BOSS) or (touched.health > 1)
		mo.momx = $ * BOSS_MOMENTUM_PERCENT / 100
		mo.momy = $ * BOSS_MOMENTUM_PERCENT / 100
	end
end, MT_PLAYER)

addHook("PostThinkFrame", function()
	for player in players.iterate()
		-- Existence check
		if not A_KRCheckExistence(player)
			continue
		end
		
		local mo = player.mo
		
		-- Cut the aircurl sound early if you're not actually curled
		if mo.krework.aircurled and not (mo.state == S_PLAY_JUMP or mo.state == S_PLAY_ROLL)
			S_StopSoundByID(mo, sfx_curl)
		end
		
		player.krework.pbuttons = player.cmd.buttons
		
		-- Reset the krework.aircurled check
		mo.krework.aircurled = false
		
		-- Reset the shield ability timing counter
		if not (player.cmd.buttons & BT_USE)
		or P_IsObjectOnGround(mo)
		or A_KRCheckNoControl(player)
		or player.krework.shieldready
			player.krework.shieldt = 0
		end
		
		-- Restore thokked flag
		if player.krework.shieldready
			player.pflags = player.krework.thokked and ($1 | PF_THOKKED) or ($1 & ~PF_THOKKED)
		end
	end
end)
