-- SETTINGS
-- Sonic's drop dash
local DROP_LV1TIME = 10
local DROP_LV2TIME = 45
local DROP_LV3TIME = 85

local DROP_LV1 = {level = 1, speed = 40*FRACUNIT, dustamt = 4}
local DROP_LV2 = {level = 2, speed = 60*FRACUNIT, dustamt = 6}
local DROP_LV3 = {level = 3, speed = 80*FRACUNIT, dustamt = 8}

-- Sonic's double jump
local DJ_SPEED = 8*FRACUNIT
local DJ_MOMZ = (19/2)*FRACUNIT

-- Sonic double jump
function A_KRSonicDoubleJump(player)
	if not A_KRCheckExistence(player)
	or not (player.mo.skin == "sonic")
	or not (player.pflags & PF_JUMPED)
	or (not player.krework.superpowers and player.powers[pw_super])
		return false
	end
	
	local mo = player.mo
	
	-- Setting PF_THOKKED prevents landing into a roll
	-- Use a custom variable instead
	if mo.krework.sonic.dbljumped
		return true
	end
	
	S_StartSound(mo, sfx_dbjump)
	
	-- Initialise vars
	local speed = FixedHypot(mo.momx, mo.momy)
	local momangle = R_PointToAngle2(0, 0, mo.momx, mo.momy)
	local djspeed = DJ_SPEED
	local djmomz = DJ_MOMZ
	local dustamt = 4
	local abilityaim = A_KRGetMoveDirection(player)
	
	-- Depending on the sharpness of your turn, you will lose speed
	local anglediff = AngleFixed(abs(momangle - abilityaim)) / FRACUNIT
	local speedpercentage = max(0, min(100, (100 - (anglediff - 50))))
	speed = $ * speedpercentage / 100
	
	-- print(AngleFixed(momangle) / FRACUNIT)
	-- print(AngleFixed(abilityaim) / FRACUNIT)
	-- print(AngleFixed(mo.angle) / FRACUNIT)
	-- print(anglediff)
	
	-- Flags, states
	mo.krework.sonic.dropcharge = 0 -- Reset drop dash timer
	player.pflags = $ | PF_JUMPED
	player.pflags = $ | PF_STARTJUMP
	mo.state = S_PLAY_JUMP
	
	-- Underwater and super speed modifiers
	if mo.eflags&MFE_UNDERWATER
		djspeed = $*2/3
		djmomz = $*2/3
	end
	if player.powers[pw_super]
		djspeed = $*3/2
		djmomz = $*3/2
		local dustamt = 6
	end
	
	-- Apply vertical thrust
	P_SetObjectMomZ(mo, djmomz, false)
	
	-- Apply horizontal thrust unless the movement controls are mostly neutral
	if (player.krework.freeaim and FixedHypot(abs(player.cmd.forwardmove), abs(player.cmd.sidemove)) > 5)
	or (not player.krework.freeaim and player.cmd.forwardmove > 5)
		if not player.krework.freeaim -- Always face forwards
			player.drawangle = mo.angle
			abilityaim = mo.angle
			P_InstaThrust(mo, mo.angle, max(djspeed, speed))
		else -- Free aim
			player.drawangle = abilityaim
			P_InstaThrust(mo, abilityaim, max(djspeed, speed))
		end
	end
	
	-- Dust
	local function r(mo, value)
		local w = mo.radius/FRACUNIT/2
		return value + P_RandomRange(-w, w) * FRACUNIT
	end
	
	for n = 1, dustamt
		local d = P_SpawnMobj(r(mo, mo.x), r(mo, mo.y), mo.z, MT_SPINDUST)
		d.extravalue1 = FRACUNIT
		d.extravalue2 = FRACUNIT
		if P_MobjFlip(mo) == -1
			d.eflags = $|MFE_VERTICALFLIP
		end
		P_SetObjectMomZ(d, FRACUNIT*2)
		local l = 180*n/dustamt
		P_InstaThrust(d, abilityaim+ANGLE_135+l*ANG1, DJ_SPEED/2)
	end
	
	mo.krework.sonic.dbljumped = true
	return true
end

function A_KRChargeDropDash(player)
	local mo = player.mo
	local charge = mo.krework.sonic.dropcharge
	
	-- Increase dropdash charge, cap at level 3 + 1 frame
	local function chargeUp()
		charge = min($ + 1, DROP_LV3TIME + 1)
	
		-- SFX for reaching a new charge level
		if charge == DROP_LV1TIME
			S_StartSound(mo, sfx_drpdsh)
		elseif charge == DROP_LV2TIME
			S_StartSound(mo, sfx_s3kab8)
		elseif charge == DROP_LV3TIME
			S_StartSound(mo, sfx_s3kabf)
		end
	end
	
	chargeUp()
	-- Charge twice as fast with sneakers
	if player.powers[pw_sneakers]
		chargeUp()
	end
	
	mo.krework.sonic.dropcharge = charge
	
	-- Dropdashing is LIKE spindashing
	if not (player.pflags & PF_ANALOGMODE)
--		player.pflags = $ | PF_STARTDASH
	else
--		player.pflags = $ & ~PF_STARTDASH
	end
	
	-- Ultra drop dash is ready
	if charge >= DROP_LV3TIME
		local trail = P_SpawnGhostMobj(mo)
		P_TeleportMove(trail, trail.x, trail.y, trail.z - 7 * FRACUNIT)
		trail.fuse = 12
		trail.state = S_THOK
		trail.scale = FRACUNIT
		trail.destscale = FRACUNIT/4
		-- trail.color = SKINCOLOR_CYAN
		-- trail.colorized = true
		
	-- No? Then super drop dash is ready
	elseif charge >= DROP_LV2TIME
		local trail = P_SpawnGhostMobj(mo)
		P_TeleportMove(trail, trail.x, trail.y, trail.z - 3 * FRACUNIT)
		trail.fuse = 8
		trail.state = S_THOK
		trail.scale = FRACUNIT * 4/5 
		trail.destscale = FRACUNIT/4
		trail.frame = TR_TRANS80|A
		
	-- No? Then regular drop dash is ready
	elseif charge >= DROP_LV1TIME
		if mo.state == S_PLAY_JUMP
			mo.state = S_DROPDASH1
		end
	end
	
	-- Always face forwards when charging dropdash
	if not player.krework.freeaim
		player.drawangle = mo.angle
	end
end

function A_KRReleaseDropDash(player)
	local mo = player.mo
	local charge = mo.krework.sonic.dropcharge
	
	if charge < DROP_LV1TIME
		return
	end
	
	local speed = FixedHypot(mo.momx, mo.momy)
	local dropdata = DROP_LV1
	
	-- Dropdash goes the way the character is facing.
	-- In simple: That's the way you're moving.
	-- In strafe/analog strafe: That's forwards.
	-- In standard controls:
	-- With freeaim off, face them forward
	if A_KRGetCtrlStyle(player) == "standard"
		-- With freeaim on, face the player in movement direction
		if player.krework.freeaim
			player.drawangle = A_KRGetMoveDirection(player)
		-- With freeaim off, face them forward
		else
			player.drawangle = mo.angle
		end
	end
	
	-- Decide which charge level you're at
	if charge >= DROP_LV2TIME
		if charge < DROP_LV3TIME
			dropdata = DROP_LV2
		else
			dropdata = DROP_LV3
		end
	end
	
	-- State change
	mo.state = S_PLAY_ROLL
	player.pflags = $1|PF_SPINNING
	
	-- Thrust
	local dropspeed = dropdata.speed
		-- Underwater and super speed modifiers
		if mo.eflags&MFE_UNDERWATER
			dropspeed = $*2/3
		end
		if player.powers[pw_super]
			dropspeed = $*3/2
		end
	
	P_InstaThrust(mo, player.drawangle, max(dropspeed, speed))
	
	-- Sounds
	S_StartSound(mo, sfx_zoom)
	if dropdata.level >= 2
		S_StartSound(mo, sfx_s3k81)
	end
	if dropdata.level == 3
		S_StartSound(mo, sfx_s3ka2)
	end
	
	-- Dust
	local dustamt = dropdata.dustamt
	
	local function r(mo, value)
		local w = mo.radius/FRACUNIT/2
		return value + P_RandomRange(-w, w) * FRACUNIT
	end
	
	for n = 1, dustamt
		local d = P_SpawnMobj(r(mo, mo.x), r(mo, mo.y), mo.z, MT_SPINDUST)
		d.extravalue1 = FRACUNIT
		d.extravalue2 = FRACUNIT
		if P_MobjFlip(mo) == -1
			d.eflags = $|MFE_VERTICALFLIP
		end
		P_SetObjectMomZ(d, FRACUNIT*2)
		local l = 90*n/dustamt
		P_InstaThrust(d, player.drawangle+ANGLE_135+l*ANG1, dropspeed/2)
	end
	
	-- Quake
	if dropdata.level == 3
		P_StartQuake(30*FRACUNIT, 5, {mo.x, mo.y, mo.z}, 200 * FRACUNIT)
	end
	
	mo.krework.sonic.dropcharge = 0
end
