-- SETTINGS
-- Initial flight lift
local FLY_ASCENDSPEED = (53/20)*FRACUNIT
-- Maximum flight upward speed (unused)
local FLY_MAXMOMZ = 10 * FRACUNIT
-- Number of tics you spend ascending
local FLY_ASCENDTICS = 7
-- Number of tics you spend ascending when swimming
local FLY_ASCENDTICS_SWIM = 4
-- Minimum time between flight boosts
local FLY_SPAMTICS = 10
-- How fast you fall at first
local FLY_HOVERSPEED = -(7/2) * FRACUNIT
-- How fast you fall when you're exhausted
local FLY_HOVERSPEED_TIRED = -(11/2) * FRACUNIT
-- How many tics it takes you to enter the exhausted state
local FLY_TIREDTIME = 55
-- How many tics it takes to drop out of the air entirely
local FLY_MAXTIME = 85

-- Tails flight activated
function A_KRTailsFly(player)
	if not A_KRCheckExistence(player)
	or not (player.mo.skin == "tails")
	or player.bot
	or not (player.pflags & PF_JUMPED)
		return false
	end
	
	local mo = player.mo
	local speed = FixedHypot(mo.momx, mo.momy)
	local momangle = R_PointToAngle2(0, 0, mo.momx, mo.momy)
	local count = mo.krework.tails.flycount
	local time = mo.krework.tails.flytime
	
	if count <= 0
	or (time > 0 and time < FLY_SPAMTICS)
		return true
	end
	
	-- Momentum
	mo.momz = $ / 2
	P_Thrust(mo, momangle + ANGLE_180, speed / 4)
	
	-- Flags, states
	mo.krework.aircurled = false
	mo.krework.tails.flytime = 0
	mo.krework.tails.flycount = $ - 1
	mo.krework.tails.flylock = FLY_SPAMTICS
	player.pflags = $ | PF_NOJUMPDAMAGE
	player.pflags = $ & ~PF_SPINNING
	mo.state = S_PLAY_FLY
	
	-- Sound effect
	if not (mo.eflags & MFE_UNDERWATER)
		S_StartSound(mo, sfx_putput)
	end
	return true
end

function A_KRTailsHover(player)
	local mo = player.mo
	local time = mo.krework.tails.flytime

	-- Declare flight-related variables
	local hoverspeed = FLY_HOVERSPEED
	local flip = P_MobjFlip(mo)
	
	-- Change state to tired to show when you can boost again
	if time >= FLY_SPAMTICS
		if mo.state == S_PLAY_FLY
			mo.state = S_PLAY_FLY_TIRED
		end
	end
	
	-- Change sound when you hit tired physics
	if time < FLY_TIREDTIME
		if time % 10 == 1
		and (mo.state == S_PLAY_FLY or mo.state == S_PLAY_FLY_TIRED)
			S_StartSound(mo, sfx_s3kba)
		end
	else
		if mo.state == S_PLAY_FLY_TIRED
			if mo.eflags & MFE_UNDERWATER
				mo.state = S_PLAY_FLY
			elseif time % 5 == 1
				S_StartSound(mo, sfx_s3kbb)
			end
		end
		
		-- Hover at tired hovering speed
		hoverspeed = FLY_HOVERSPEED_TIRED
	end
	
	-- Decide on vertboost duration
	local ascendtics = FLY_ASCENDTICS
	if mo.eflags & MFE_UNDERWATER
		ascendtics = FLY_ASCENDTICS_SWIM
	end
	local speed = FixedHypot(mo.momx, mo.momy)
	
	//print(speed/FRACUNIT)
	
	-- Fall no faster than your hover speed
	if (flip == 1 and mo.momz < hoverspeed) or (flip == -1 and mo.momz > -hoverspeed)
		P_SetObjectMomZ(mo, hoverspeed, false)
	end
	//P_SetObjectMomZ(mo, min(flippedmomz, hoverspeed))
	
	-- If you're still in vertboost, apply boost
	if (time <= ascendtics)
		-- Calculate boost to apply
		local ascendspeed = FLY_ASCENDSPEED
		if mo.eflags&MFE_UNDERWATER
			ascendspeed = $ * 2/3
		end
		
		-- Dust clouds whoe color represents the number of times Tails has boosted
		if time % 2 == 0 and not (mo.eflags&MFE_UNDERWATER) and mo.krework.tails.flycount < 3
			local tails = player.followmobj
			
			local angle = (player.mo.angle + (tails.frame * ANGLE_45))
			local radius = tails.radius * 5 / 4
			
			local dust1x = FixedMul(cos(angle), radius)
			local dust1y = FixedMul(sin(angle), radius)
			local dust2x = FixedMul(cos(angle+ANGLE_180), radius)
			local dust2y = FixedMul(sin(angle+ANGLE_180), radius)
			
			local dusts = {[2] = SKINCOLOR_YELLOW, [1] = SKINCOLOR_ORANGE, [0] = SKINCOLOR_RED}
			local dcolor = dusts[mo.krework.tails.flycount] or SKINCOLOR_EMERALD
			
			local d1 = P_SpawnMobjFromMobj(tails, dust1x, dust1y, (tails.height*4/5), MT_SPINDUST)
			local d2 = P_SpawnMobjFromMobj(tails, dust2x, dust2y, (tails.height*4/5), MT_SPINDUST)
			d1.color, d2.color = dcolor, dcolor
			d1.colorized, d2.colorized = true, true
		end
		
		P_SetObjectMomZ(mo, ascendspeed, true)
	end
	
	if time >= FLY_MAXTIME
	and (mo.state == S_PLAY_FLY or mo.state == S_PLAY_FLY_TIRED)
		mo.krework.tails.flytime = 0
		mo.state = S_PLAY_FALL
	else
		mo.krework.tails.flytime = $ + 1
	end
end
